@@ -2,13 +2,13 @@ from typing    import Callable, Dict, List, Optional, Set, Tuple
 from irctokens import build, Hostmask, Line, StatefulDecoder, StatefulEncoder
 from pendulum  import from_timestamp, now
 
-from .named        import Named
 from .user         import User
 from .channel      import Channel
 from .channel_user import ChannelUser
 from .isupport     import ISupport
 from .decorators   import handler_decorator
 from .casemap      import casefold
+from .names        import Name
 from .emit         import *
 from .numerics     import *
 
@@ -23,7 +23,7 @@ class ServerDisconnectedException(ServerException):
 WHO_TYPE = "524" # randomly generated
 TYPE_EMIT = Optional[Emit]
 
-class Server(Named):
+class Server(object):
     def __init__(self, name: str):
         self.name = name
 
@@ -84,7 +84,7 @@ class Server(Named):
     def has_user(self, nickname: str) -> bool:
         return self.casefold(nickname) in self.users
     def _add_user(self, nickname: str, nickname_lower: str):
-        user = self._create_user(nickname, nickname_lower)
+        user = self.create_user(Name(nickname, nickname_lower))
         self.users[nickname_lower] = user
 
     def is_channel(self, target: str) -> bool:
@@ -94,22 +94,16 @@ class Server(Named):
     def get_channel(self, name: str) -> Optional[Channel]:
         return self.channels.get(self.casefold(name), None)
 
-    def create_user(self) -> User:
-        return User()
-    def _create_user(self, nickname: str, nickname_lower: str) -> User:
-        user = self.create_user()
-        user.set_nickname(nickname, nickname_lower)
-        return user
+    def create_user(self, nickname: Name) -> User:
+        return User(nickname)
 
-    def create_channel(self) -> Channel:
-        return Channel()
-    def _create_channel(self, name: str, name_lower: str) -> Channel:
-        channel = self.create_channel()
-        channel.set_name(name, name_lower)
-        return channel
+    def create_channel(self, name: Name) -> Channel:
+        return Channel(name)
 
     def _user_join(self, channel: Channel, user: User) -> ChannelUser:
-        channel_user = ChannelUser()
+        channel_user = ChannelUser(
+            user.get_name(),
+            channel.get_name())
 
         user.channels.add(self.casefold(channel.name))
         channel.users[user.nickname_lower] = channel_user
@@ -139,7 +133,7 @@ class Server(Named):
     @line_handler(RPL_ISUPPORT)
     # https://defs.ircdocs.horse/defs/isupport.html
     def _handle_ISUPPORT(self, line: Line) -> Emit:
-        self.isupport.tokens(line.params[1:-1])
+        self.isupport.from_tokens(line.params[1:-1])
         return self._emit()
 
     @line_handler(RPL_MOTDSTART)
@@ -160,29 +154,28 @@ class Server(Named):
 
     @line_handler("NICK")
     def _handle_NICK(self, line: Line) -> Emit:
-        new_nickname = line.params[0]
-        nickname_lower = self.casefold(line.hostmask.nickname)
+        new_nickname       = line.params[0]
+        new_nickname_lower = self.casefold(new_nickname)
+        nickname_lower     = self.casefold(line.hostmask.nickname)
 
         emit = self._emit()
 
         if nickname_lower in self.users:
             user = self.users.pop(nickname_lower)
             emit.user = user
-
-            old_nickname_lower = user.nickname_lower
-            new_nickname_lower = self.casefold(new_nickname)
-            user.set_nickname(new_nickname, new_nickname_lower)
+            user.change_nickname(new_nickname, new_nickname_lower)
             self.users[new_nickname_lower] = user
+
             for channel_lower in user.channels:
                 channel = self.channels[channel_lower]
-                channel_user = channel.users.pop(old_nickname_lower)
+                channel_user = channel.users.pop(nickname_lower)
                 channel.users[user.nickname_lower] = channel_user
 
         if nickname_lower == self.nickname_lower:
             emit.self = True
 
-            self.nickname = new_nickname
-            self.nickname_lower = self.casefold(new_nickname)
+            self.nickname       = new_nickname
+            self.nickname_lower = new_nickname_lower
         return emit
 
     @line_handler("JOIN")
@@ -199,8 +192,9 @@ class Server(Named):
         if nickname_lower == self.nickname_lower:
             emit.self = True
             if not channel_lower in self.channels:
-                channel = self._create_channel(line.params[0],
-                    channel_lower)
+                channel = self.create_channel(
+                    Name(line.params[0], channel_lower)
+                )
                 self.channels[channel_lower] = channel
 
             self._self_hostmask(line.hostmask)
@@ -288,8 +282,9 @@ class Server(Named):
             if kicker_lower in self.users:
                 emit.user_source = self.users[kicker_lower]
             else:
-                emit.user_source = self._create_user(line.hostmask.nickname,
-                    kicker_lower)
+                emit.user_source = self.create_user(
+                    Name(line.hostmask.nickname, kicker_lower)
+                )
 
         return emit
 
@@ -421,8 +416,7 @@ class Server(Named):
             char = mode[1]
             arg: Optional[str] = None
 
-            list_mode = char in self.isupport.chanmodes.list_modes
-            if char in self.isupport.prefix.modes:
+            if char in self.isupport.prefix.modes: # a user's status
                 arg = params.pop(0)
                 nickname_lower = self.casefold(arg)
 
@@ -434,22 +428,26 @@ class Server(Named):
                             channel_user.modes.append(char)
                     elif char in channel_user.modes:
                         channel_user.modes.remove(char)
-            elif add and (
-                    list_mode or
-                    char in self.isupport.chanmodes.setting_b_modes or
-                    char in self.isupport.chanmodes.setting_c_modes):
-                arg = params.pop(0)
-                channel.add_mode(char, arg, list_mode)
-            elif not add and (
-                    list_mode or
-                    char in self.isupport.chanmodes.setting_b_modes):
-                arg = params.pop(0)
-                channel.remove_mode(char, arg)
             elif add:
-                channel.add_mode(char, None, False)
-            else:
-                channel.remove_mode(char, None)
+                if char in self.isupport.chanmodes.a_modes:
+                    arg = params.pop(0)
+                    channel.add_mode(char, arg, True)
+                elif (char in self.isupport.chanmodes.b_modes or
+                        char in self.isupport.chanmodes.c_modes):
+                    arg = params.pop(0)
+                    channel.add_mode(char, arg, False)
+                else:
+                    channel.add_mode(char, None, False)
+            else: # remove
+                if (char in self.isupport.chanmodes.a_modes or
+                        char in self.isupport.chanmodes.b_modes):
+                    arg = params.pop(0)
+                    channel.remove_mode(char, arg)
+                else:
+                    channel.remove_mode(char, None)
+
             tokens.append((mode, arg))
+
         return tokens
 
     @line_handler("MODE")
@@ -533,7 +531,9 @@ class Server(Named):
         if nickname_lower in self.users:
             user = self.users[nickname_lower]
         else:
-            user = self._create_user(line.hostmask.nickname, nickname_lower)
+            user = self.create_user(
+                Name(line.hostmask.nickname, nickname_lower)
+            )
         emit.user = user
 
         if line.hostmask.username:
