@@ -1,11 +1,10 @@
 from typing import Optional, Set
-from .named import Named
+from .names import Name
 
-class User(Named):
-    nickname:       str
-    nickname_lower: str
+class User(object):
+    def __init__(self, nickname: Name):
+        self._nickname = nickname
 
-    def __init__(self):
         self.username: Optional[str] = None
         self.hostname: Optional[str] = None
         self.realname: Optional[str] = None
@@ -16,10 +15,20 @@ class User(Named):
     def __repr__(self) -> str:
         return f"User(nickname={self.nickname!r})"
 
-    def set_nickname(self, nickname: str,
-            nickname_lower: str):
-        self.nickname       = nickname
-        self.nickname_lower = nickname_lower
+    def get_name(self) -> Name:
+        return self._nickname
+    @property
+    def nickname(self) -> str:
+        return self._nickname.normal
+    @property
+    def nickname_lower(self) -> str:
+        return self._nickname.folded
+
+    def change_nickname(self,
+            normal: str,
+            folded: str):
+        self._nickname.normal = normal
+        self._nickname.folded = folded
 
     def hostmask(self) -> str:
         hostmask = self.nickname
